import logging
from xmlrpc import client

from .gettext import _

_logger = logging.getLogger(__name__)


class Sender:
    """the Sender class is responsible to use whatever method
    to upload data to Odoo and return a result flow
    """

    def __init__(
        self, mode, oeprotocol, oeserver, oeport, oeuser, oepass, oedb, builder
    ):
        self.mode = mode
        self.oeprotocol = oeprotocol
        self.oeserver = oeserver
        self.oeport = oeport
        self.oeuser = oeuser
        self.oepass = oepass
        self.oedb = oedb
        self.builder = builder

        url = f"{self.oeprotocol}://{self.oeserver}:{self.oeport}"
        login_url = f"{url}/xmlrpc/2/common"
        print(_("Accessing {}").format(login_url))
        sock = client.ServerProxy(login_url)
        self.uid = sock.authenticate(self.oedb, self.oeuser, self.oepass, {})
        common_url = f"{url}/xmlrpc/2/object"

        self.sock = client.ServerProxy(common_url, allow_none=True)

    def send(self):
        for item in self.builder.build():
            mode = self.mode
            try:

                if mode == "auto":
                    # Update if there’s an id, create if none
                    if not item.get("id", 0) == 0:
                        mode = "write"
                    else:
                        mode = "create"

                if mode == "create":
                    yield (
                        True,
                        self.sock.execute_kw(
                            self.oedb,
                            self.uid,
                            self.oepass,
                            self.builder.itemtype,
                            mode,
                            [item],
                        ),
                    )

                elif mode == "write":
                    # if you ask for a write operation you should give us
                    # a proper db id for your item
                    item_id = item.pop("id")
                    yield (
                        True,
                        self.sock.execute_kw(
                            self.oedb,
                            self.uid,
                            self.oepass,
                            self.builder.itemtype,
                            mode,
                            [[item_id], item],
                        ),
                    )

                elif mode == "unlink":
                    item_id = item.pop("id")
                    yield (
                        True,
                        self.sock.execute_kw(
                            self.oedb,
                            self.uid,
                            self.oepass,
                            self.builder.itemtype,
                            mode,
                            [[item_id]],
                        ),
                    )

                else:
                    raise NotImplementedError(
                        _("The method {} is not yet supported.").format(self.mode)
                    )

            except client.Fault as e:
                yield False, (e.faultString, item)
